<?php
namespace Delo;

class Settings
{
    /**
     * Все настройки
     * @return array
     */
    public function settingFields()
    {
        return [
            'status' => [
                'filter' => [
                    'int',
                ],
                'validate' => [
                    'enable',
                ],
                'type'   => 'switch',
            ],
            'title' => [
                'filter' => [
                    'default',
                ],
                'validate' => [
                    'required',
                ],
                'type' => 'text',
            ],
            'merch_id' => [
                'filter' => [
                    'default',
                ],
                'validate' => [
                    'required',
                    'pcid',
                    'pcidLength',
                ],
                'type' => 'text',
            ],
            'purchase_account_amount_id' => [
                'filter' => [
                    'default',
                ],
                'validate' => [
                    'required',
                    'pcid',
                    'pcidLength',
                ],
                'type' => 'text',
            ],
            'back_url_f' => [
                'filter' => [
                    'default',
                ],
                'validate' => [
                    'required',
                    'https',
                    'url',
                ],
                'type' => 'text',
            ],
            'url' => [
                'filter' => [
                    'default',
                ],
                'validate' => [
                    'required',
                    'https',
                    'urlDelo',
                ],
                'type' => 'text',
            ],
            'lang' => [
                'filter' => [
                    'default',
                ],
                'validate' => [
                    'required',
                    'lang',
                ],
                'type'   => 'select',
                'values' => [
                    'RU' => 'RU',
                    'EN' => 'EN',
                ],
            ],
            'currency' => [
                'filter' => [
                    'default',
                ],
                'validate' => [
                    'required',
                    'int',
                    'currency',
                ],
                'type'   => 'select',
                'values' => [
                    '643' => '643',
                ],
            ],
            'info' => [
                'filter' => [
                    'default',
                ],
                'validate' => [
                ],
                'type'   => 'textarea',
            ],
            'order_status_after' => [
                'filter' => [
                    'int',
                ],
                'validate' => [
                    'status',
                ],
                'type' => 'select',
            ],
            'order_status_failed' => [
                'filter' => [
                    'int',
                ],
                'validate' => [
                    'status',
                ],
                'type' => 'select',
            ],
            'check_signature' => [
                'filter' => [
                    'int',
                ],
                'validate' => [
                    'enable',
                ],
                'type' => 'switch',
            ],
            'debug_mode' => [
                'filter' => [
                    'int',
                ],
                'validate' => [
                    'enable',
                ],
                'type' => 'switch',
            ],
        ];
    }

    /**
     * Defaults значения настроек
     * @return array
     */
    public function defaultValues()
    {
        $baseUrl = _PS_BASE_URL_SSL_ . __PS_BASE_URI__;
        $defaultSettings = [
            'title'           => 'Оплата картой банка',
            'back_url_f'      => $baseUrl . '',
            'url'             => isset($_SERVER['DELOBANK_ENVIRONMENT']) ? 'https://testmpi.myskb.ru/payment/start.wsm' : 'https://mpi.skbbank.ru/payment/start.wsm',
            'lang'            => 'RU',
            'currency'        => 643,
            'status'          => 0,
            'check_signature' => 1,
            'debug_mode'      => 0,
        ];
        // Дополнить те у которых нет default значении, пустыми значениями, чтобы в БД заранее были записи для каждой настройки
        $fields = $this->settingFields();
        foreach ($fields as $fieldName => $fieldSetting) {
            if (!isset($defaultSettings[$fieldName])) {
                $defaultSettings[$fieldName] = '';
            }
        }
        return $defaultSettings;
    }

    /**
     * @param string $val
     * @return int
     */
    private function intFilter($val)
    {
        return (int)$val;
    }

    /**
     * @param string $val
     * @return string
     */
    private function defaultFilter($val)
    {
        return trim($val);
    }

    /**
     * @param string $val
     * @return string
     */
    private function htmlspecialcharsFilter($val)
    {
        return htmlspecialchars($val, ENT_QUOTES | ENT_HTML5);
    }

    /**
     * Проверка, что поле обязательно задано, если не задано возвращает ошибку, иначе null
     * @param string $val
     * @return string|null
     */
    private function requiredValidate($val)
    {
        $error = null;
        if (empty($val)) {
            $error = $this->language->get('required_error');
        }
        return $error;
    }

    /**
     * Проверка, что путь до файла существует
     * Основан на проверке существования файла
     * @param string $path путь до файла
     * @return string|null
     */
    private function certValidate($path)
    {
        if (empty($path)) {
            return null;
        }
        $realPath = realpath(DIR_APPLICATION . '/../' . $path);
        if (!is_readable($realPath)) {
            return $this->language->get('cert_error_text');
        }
        return null;
    }

    /**
     * Ищет запрещенные символы в PCID
     * @param string $val
     * @return string
     */
    public function pcidValidate($val)
    {
        $error = null;
        if (preg_match('/[^a-z0-9]/i', $val)) {
            $error = $this->language->get('pcid_error_validate_text');
        }
        return $error;
    }

    /**
     * Проверка длины PCID
     * @param string $val
     * @return string
     */
    public function pcidLengthValidate($val)
    {
        $error = null;
        if (mb_strlen($val) != 32) {
            $error = $this->language->get('pcid_error_validate_length');
        }
        return $error;
    }

    /**
     * Проверяет есть ли язык среди возможных
     * @param string $val
     * @return string|null
     */
    private function langValidate($val)
    {
        $error = null;
        $langs = $this->optionsLang();
        if (!isset($langs[$val])) {
            $error = 'Язык "' . $val . '" не существует';
        }
        return $error;
    }

    /**
     * Проверяет есть ли валюта среди возможных
     * @param int $val
     * @return string|null
     */
    private function currencyValidate($val)
    {
        $error = null;
        $currencyArr = $this->optionsCurrency();
        if (!isset($currencyArr[$val])) {
            $error = $this->language->get('currency_ot exist_error');
        }
        return $error;
    }

    /**
     * Проверяет что в значении только числа
     * @param int $val
     * @return string|null
     */
    private function intValidate($val)
    {
        $error = null;
        $intVal = (string)(int)$val;
        if ($intVal != $val) {
            $error = $this->language->get('int_error');
        }
        return $error;
    }

    /**
     * Проверяет что это url
     * @param string $val
     * @return string|null
     */
    private function urlDeloValidate($val)
    {
        $error = null;
        $need = [
            'https://mpi.skbbank.ru/payment/start.wsm' => 1,
            'https://testmpi.myskb.ru/payment/start.wsm' => 1,
        ];
        if (!isset($need[$val])) {
            $error = $this->language->get('url_delo_error');
        }
        return $error;
    }

    /**
     * Проверяет что это url
     * @param string $url
     * @return string|null
     */
    private function urlValidate($url)
    {
        $error = null;
        if (!filter_var($url, FILTER_VALIDATE_URL)) {
            $error = $this->language->get('url_our_error');
        }
        return $error;
    }

    /**
     * Проверяет что это url
     * @param string $val
     * @return string|null
     */
    private function httpsValidate($val)
    {
        $error = null;
        if (strpos($val, 'https') === false) {
            $error = $this->language->get('https_error');
        }
        return $error;
    }

    /**
     * Вкл/выкл способа оплаты
     * @param int $val
     * @return string|null
     */
    private function enableValidate($val)
    {
        $error = null;
        $need = [
            0 => 'выключено',
            1 => 'включено',
        ];
        if (!isset($need[$val])) {
            $error = $this->language->get('enable_error');
        }
        return $error;
    }

    /**
     * Валидация статусов заказов
     * @param int $statusId
     * @return string|null
     */
    private function statusValidate($statusId)
    {
        if (!empty($this->modelOrderStatus)) {
            $orderStatuses = $this->modelOrderStatus->getOrderStatuses();
            $newOrderStatus = [];
            foreach ($orderStatuses as $status) {
                $newOrderStatus[$status['order_status_id']] = $status['name'];
            }
            $orderStatuses = $newOrderStatus;
        } else {
            $orderStatuses = [];
        }
        $error = null;
        if (!isset($orderStatuses[$statusId])) {
            $error = $this->language->get('status_error');
        }
        return $error;
    }
}
